@@ -15,6 +15,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 """
 SQLAlchemy models for cinder data.
 """
@@ -22,10 +23,12 @@ SQLAlchemy models for cinder data.
 from oslo_config import cfg
 from oslo_db.sqlalchemy import models
 from oslo_utils import timeutils
+from sqlalchemy import and_, func, select
+from sqlalchemy import bindparam
 from sqlalchemy import Column, Integer, String, Text, schema
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy import ForeignKey, DateTime, Boolean
-from sqlalchemy.orm import relationship, backref, validates
+from sqlalchemy import ForeignKey, DateTime, Boolean, UniqueConstraint
+from sqlalchemy.orm import backref, column_property, relationship, validates
 
 
 CONF = cfg.CONF
@@ -44,11 +47,17 @@ class CinderBase(models.TimestampMixin,
     deleted = Column(Boolean, default=False)
     metadata = None
 
+    @staticmethod
+    def delete_values():
+        return {'deleted': True,
+                'deleted_at': timeutils.utcnow()}
+
     def delete(self, session):
         """Delete this object."""
-        self.deleted = True
-        self.deleted_at = timeutils.utcnow()
+        updated_values = self.delete_values()
+        self.update(updated_values)
         self.save(session=session)
+        return updated_values
 
 
 class Service(BASE, CinderBase):
@@ -56,8 +65,13 @@ class Service(BASE, CinderBase):
 
     __tablename__ = 'services'
     id = Column(Integer, primary_key=True)
+    cluster_name = Column(String(255), nullable=True)
     host = Column(String(255))  # , ForeignKey('hosts.id'))
     binary = Column(String(255))
+    # We want to overwrite default updated_at definition so we timestamp at
+    # creation as well, so we only need to check updated_at for the heartbeat
+    updated_at = Column(DateTime, default=timeutils.utcnow,
+                        onupdate=timeutils.utcnow)
     topic = Column(String(255))
     report_count = Column(Integer, nullable=False, default=0)
     disabled = Column(Boolean, default=False)
@@ -69,13 +83,73 @@ class Service(BASE, CinderBase):
     # periodic updates
     modified_at = Column(DateTime)
 
-    # Version columns to support rolling upgrade.
-    # Current version is what the service is running now (i.e. minimum).
-    # Available version is what the service can support (i.e. max).
+    # Version columns to support rolling upgrade. These report the max RPC API
+    # and objects versions that the manager of the service is able to support.
     rpc_current_version = Column(String(36))
-    rpc_available_version = Column(String(36))
     object_current_version = Column(String(36))
-    object_available_version = Column(String(36))
+
+    # replication_status can be: enabled, disabled, not-capable, error,
+    # failed-over or not-configured
+    replication_status = Column(String(255), default="not-capable")
+    active_backend_id = Column(String(255))
+    frozen = Column(Boolean, nullable=False, default=False)
+
+    cluster = relationship('Cluster',
+                           backref='services',
+                           foreign_keys=cluster_name,
+                           primaryjoin='and_('
+                                       'Service.cluster_name == Cluster.name,'
+                                       'Service.deleted == False)')
+
+
+class Cluster(BASE, CinderBase):
+    """Represents a cluster of hosts."""
+    __tablename__ = 'clusters'
+    # To remove potential races on creation we have a constraint set on name
+    # and race_preventer fields, and we set value on creation to 0, so 2
+    # clusters with the same name will fail this constraint.  On deletion we
+    # change this field to the same value as the id which will be unique and
+    # will not conflict with the creation of another cluster with the same
+    # name.
+    __table_args__ = (UniqueConstraint('name', 'binary', 'race_preventer'),)
+
+    id = Column(Integer, primary_key=True)
+    # NOTE(geguileo): Name is constructed in the same way that Server.host but
+    # using cluster configuration option instead of host.
+    name = Column(String(255), nullable=False)
+    binary = Column(String(255), nullable=False)
+    disabled = Column(Boolean, default=False)
+    disabled_reason = Column(String(255))
+    race_preventer = Column(Integer, nullable=False, default=0)
+
+    # Last heartbeat reported by any of the services of this cluster.  This is
+    # not deferred since we always want to load this field.
+    last_heartbeat = column_property(
+        select([func.max(Service.updated_at)]).
+        where(and_(Service.cluster_name == name, ~Service.deleted)).
+        correlate_except(Service), deferred=False)
+
+    # Number of existing services for this cluster
+    num_hosts = column_property(
+        select([func.count(Service.id)]).
+        where(and_(Service.cluster_name == name, ~Service.deleted)).
+        correlate_except(Service),
+        group='services_summary', deferred=True)
+
+    # Number of services that are down for this cluster
+    num_down_hosts = column_property(
+        select([func.count(Service.id)]).
+        where(and_(Service.cluster_name == name,
+                   ~Service.deleted,
+                   Service.updated_at < bindparam('expired'))).
+        correlate_except(Service),
+        group='services_summary', deferred=True)
+
+    @staticmethod
+    def delete_values():
+        return {'race_preventer': Cluster.id,
+                'deleted': True,
+                'deleted_at': timeutils.utcnow()}
 
 
 class ConsistencyGroup(BASE, CinderBase):
@@ -86,6 +160,7 @@ class ConsistencyGroup(BASE, CinderBase):
     user_id = Column(String(255), nullable=False)
     project_id = Column(String(255), nullable=False)
 
+    cluster_name = Column(String(255), nullable=True)
     host = Column(String(255))
     availability_zone = Column(String(255))
     name = Column(String(255))
@@ -96,6 +171,25 @@ class ConsistencyGroup(BASE, CinderBase):
     source_cgid = Column(String(36))
 
 
+class Group(BASE, CinderBase):
+    """Represents a generic volume group."""
+    __tablename__ = 'groups'
+    id = Column(String(36), primary_key=True)
+
+    user_id = Column(String(255), nullable=False)
+    project_id = Column(String(255), nullable=False)
+
+    cluster_name = Column(String(255))
+    host = Column(String(255))
+    availability_zone = Column(String(255))
+    name = Column(String(255))
+    description = Column(String(255))
+    status = Column(String(255))
+    group_type_id = Column(String(36))
+    group_snapshot_id = Column(String(36))
+    source_group_id = Column(String(36))
+
+
 class Cgsnapshot(BASE, CinderBase):
     """Represents a cgsnapshot."""
     __tablename__ = 'cgsnapshots'
@@ -116,6 +210,27 @@ class Cgsnapshot(BASE, CinderBase):
         primaryjoin='Cgsnapshot.consistencygroup_id == ConsistencyGroup.id')
 
 
+class GroupSnapshot(BASE, CinderBase):
+    """Represents a group snapshot."""
+    __tablename__ = 'group_snapshots'
+    id = Column(String(36), primary_key=True)
+
+    group_id = Column(String(36), nullable=False)
+    user_id = Column(String(255))
+    project_id = Column(String(255))
+
+    name = Column(String(255))
+    description = Column(String(255))
+    status = Column(String(255))
+    group_type_id = Column(String(36))
+
+    group = relationship(
+        Group,
+        backref="group_snapshots",
+        foreign_keys=group_id,
+        primaryjoin='GroupSnapshot.group_id == Group.id')
+
+
 class Volume(BASE, CinderBase):
     """Represents a block storage device that can be attached to a vm."""
     __tablename__ = 'volumes'
@@ -140,6 +255,7 @@ class Volume(BASE, CinderBase):
 
     snapshot_id = Column(String(36))
 
+    cluster_name = Column(String(255), nullable=True)
     host = Column(String(255))  # , ForeignKey('hosts.id'))
     size = Column(Integer)
     availability_zone = Column(String(255))  # TODO(vish): foreign key?
@@ -164,8 +280,8 @@ class Volume(BASE, CinderBase):
     encryption_key_id = Column(String(36))
 
     consistencygroup_id = Column(String(36))
+    group_id = Column(String(36))
 
-    deleted = Column(Boolean, default=False)
     bootable = Column(Boolean, default=False)
     multiattach = Column(Boolean, default=False)
 
@@ -181,6 +297,12 @@ class Volume(BASE, CinderBase):
         foreign_keys=consistencygroup_id,
         primaryjoin='Volume.consistencygroup_id == ConsistencyGroup.id')
 
+    group = relationship(
+        Group,
+        backref="volumes",
+        foreign_keys=group_id,
+        primaryjoin='Volume.group_id == Group.id')
+
 
 class VolumeMetadata(BASE, CinderBase):
     """Represents a metadata key/value pair for a volume."""
@@ -248,6 +370,42 @@ class VolumeTypes(BASE, CinderBase):
                            'VolumeTypes.deleted == False)')
 
 
+class GroupTypes(BASE, CinderBase):
+    """Represent possible group_types of groups offered."""
+    __tablename__ = "group_types"
+    id = Column(String(36), primary_key=True)
+    name = Column(String(255))
+    description = Column(String(255))
+    is_public = Column(Boolean, default=True)
+    groups = relationship(Group,
+                          backref=backref('group_type', uselist=False),
+                          foreign_keys=id,
+                          primaryjoin='and_('
+                          'Group.group_type_id == GroupTypes.id, '
+                          'GroupTypes.deleted == False)')
+
+
+class GroupVolumeTypeMapping(BASE, CinderBase):
+    """Represent mapping between groups and volume_types."""
+    __tablename__ = "group_volume_type_mapping"
+    id = Column(Integer, primary_key=True, nullable=False)
+    volume_type_id = Column(String(36),
+                            ForeignKey('volume_types.id'),
+                            nullable=False)
+    group_id = Column(String(36),
+                      ForeignKey('groups.id'),
+                      nullable=False)
+
+    group = relationship(
+        Group,
+        backref="volume_types",
+        foreign_keys=group_id,
+        primaryjoin='and_('
+        'GroupVolumeTypeMapping.group_id == Group.id,'
+        'GroupVolumeTypeMapping.deleted == False)'
+    )
+
+
 class VolumeTypeProjects(BASE, CinderBase):
     """Represent projects associated volume_types."""
     __tablename__ = "volume_type_projects"
@@ -259,6 +417,7 @@ class VolumeTypeProjects(BASE, CinderBase):
     volume_type_id = Column(Integer, ForeignKey('volume_types.id'),
                             nullable=False)
     project_id = Column(String(255))
+    deleted = Column(Integer, default=0)
 
     volume_type = relationship(
         VolumeTypes,
@@ -266,7 +425,29 @@ class VolumeTypeProjects(BASE, CinderBase):
         foreign_keys=volume_type_id,
         primaryjoin='and_('
         'VolumeTypeProjects.volume_type_id == VolumeTypes.id,'
-        'VolumeTypeProjects.deleted == False)')
+        'VolumeTypeProjects.deleted == 0)')
+
+
+class GroupTypeProjects(BASE, CinderBase):
+    """Represent projects associated group_types."""
+    __tablename__ = "group_type_projects"
+    __table_args__ = (schema.UniqueConstraint(
+        "group_type_id", "project_id", "deleted",
+        name="uniq_group_type_projects0group_type_id0project_id0deleted"),
+    )
+    id = Column(Integer, primary_key=True)
+    group_type_id = Column(Integer, ForeignKey('group_types.id'),
+                           nullable=False)
+    project_id = Column(String(255))
+    deleted = Column(Integer, default=0)
+
+    group_type = relationship(
+        GroupTypes,
+        backref="projects",
+        foreign_keys=group_type_id,
+        primaryjoin='and_('
+        'GroupTypeProjects.group_type_id == GroupTypes.id,'
+        'GroupTypeProjects.deleted == 0)')
 
 
 class VolumeTypeExtraSpecs(BASE, CinderBase):
@@ -288,38 +469,59 @@ class VolumeTypeExtraSpecs(BASE, CinderBase):
     )
 
 
+class GroupTypeSpecs(BASE, CinderBase):
+    """Represents additional specs as key/value pairs for a group_type."""
+    __tablename__ = 'group_type_specs'
+    id = Column(Integer, primary_key=True)
+    key = Column(String(255))
+    value = Column(String(255))
+    group_type_id = Column(String(36),
+                           ForeignKey('group_types.id'),
+                           nullable=False)
+    group_type = relationship(
+        GroupTypes,
+        backref="group_specs",
+        foreign_keys=group_type_id,
+        primaryjoin='and_('
+        'GroupTypeSpecs.group_type_id == GroupTypes.id,'
+        'GroupTypeSpecs.deleted == False)'
+    )
+
+
 class QualityOfServiceSpecs(BASE, CinderBase):
     """Represents QoS specs as key/value pairs.
 
     QoS specs is standalone entity that can be associated/disassociated
     with volume types (one to many relation).  Adjacency list relationship
     pattern is used in this model in order to represent following hierarchical
-    data with in flat table, e.g, following structure
+    data with in flat table, e.g, following structure:
 
-    qos-specs-1  'Rate-Limit'
-         |
-         +------>  consumer = 'front-end'
-         +------>  total_bytes_sec = 1048576
-         +------>  total_iops_sec = 500
+    .. code-block:: none
 
-    qos-specs-2  'QoS_Level1'
-         |
-         +------>  consumer = 'back-end'
-         +------>  max-iops =  1000
-         +------>  min-iops = 200
+      qos-specs-1  'Rate-Limit'
+           |
+           +------>  consumer = 'front-end'
+           +------>  total_bytes_sec = 1048576
+           +------>  total_iops_sec = 500
 
-    is represented by:
+      qos-specs-2  'QoS_Level1'
+           |
+           +------>  consumer = 'back-end'
+           +------>  max-iops =  1000
+           +------>  min-iops = 200
 
-      id       specs_id       key                  value
-    ------     --------   -------------            -----
-    UUID-1     NULL       QoSSpec_Name           Rate-Limit
-    UUID-2     UUID-1       consumer             front-end
-    UUID-3     UUID-1     total_bytes_sec        1048576
-    UUID-4     UUID-1     total_iops_sec           500
-    UUID-5     NULL       QoSSpec_Name           QoS_Level1
-    UUID-6     UUID-5       consumer             back-end
-    UUID-7     UUID-5       max-iops               1000
-    UUID-8     UUID-5       min-iops               200
+      is represented by:
+
+        id       specs_id       key                  value
+      ------     --------   -------------            -----
+      UUID-1     NULL       QoSSpec_Name           Rate-Limit
+      UUID-2     UUID-1       consumer             front-end
+      UUID-3     UUID-1     total_bytes_sec        1048576
+      UUID-4     UUID-1     total_iops_sec           500
+      UUID-5     NULL       QoSSpec_Name           QoS_Level1
+      UUID-6     UUID-5       consumer             back-end
+      UUID-7     UUID-5       max-iops               1000
+      UUID-8     UUID-5       min-iops               200
     """
     __tablename__ = 'quality_of_service_specs'
     id = Column(String(36), primary_key=True)
@@ -423,7 +625,8 @@ class Reservation(BASE, CinderBase):
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36), nullable=False)
 
-    usage_id = Column(Integer, ForeignKey('quota_usages.id'), nullable=False)
+    usage_id = Column(Integer, ForeignKey('quota_usages.id'), nullable=True)
+    allocated_id = Column(Integer, ForeignKey('quotas.id'), nullable=True)
 
     project_id = Column(String(255), index=True)
     resource = Column(String(255))
@@ -436,6 +639,10 @@ class Reservation(BASE, CinderBase):
         foreign_keys=usage_id,
         primaryjoin='and_(Reservation.usage_id == QuotaUsage.id,'
                     'QuotaUsage.deleted == 0)')
+    quota = relationship(
+        "Quota",
+        foreign_keys=allocated_id,
+        primaryjoin='and_(Reservation.allocated_id == Quota.id)')
 
 
 class Snapshot(BASE, CinderBase):
@@ -456,6 +663,7 @@ class Snapshot(BASE, CinderBase):
 
     volume_id = Column(String(36))
     cgsnapshot_id = Column(String(36))
+    group_snapshot_id = Column(String(36))
     status = Column(String(255))
     progress = Column(String(255))
     volume_size = Column(Integer)
@@ -480,6 +688,12 @@ class Snapshot(BASE, CinderBase):
         foreign_keys=cgsnapshot_id,
         primaryjoin='Snapshot.cgsnapshot_id == Cgsnapshot.id')
 
+    group_snapshot = relationship(
+        GroupSnapshot,
+        backref="snapshots",
+        foreign_keys=group_snapshot_id,
+        primaryjoin='Snapshot.group_snapshot_id == GroupSnapshot.id')
+
 
 class SnapshotMetadata(BASE, CinderBase):
     """Represents a metadata key/value pair for a snapshot."""
@@ -497,22 +711,6 @@ class SnapshotMetadata(BASE, CinderBase):
                             'SnapshotMetadata.deleted == False)')
 
 
-class IscsiTarget(BASE, CinderBase):
-    """Represents an iscsi target for a given host."""
-    __tablename__ = 'iscsi_targets'
-    __table_args__ = (schema.UniqueConstraint("target_num", "host"),
-                      {'mysql_engine': 'InnoDB'})
-    id = Column(Integer, primary_key=True)
-    target_num = Column(Integer)
-    host = Column(String(255))
-    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=True)
-    volume = relationship(Volume,
-                          backref=backref('iscsi_target', uselist=False),
-                          foreign_keys=volume_id,
-                          primaryjoin='and_(IscsiTarget.volume_id==Volume.id,'
-                          'IscsiTarget.deleted==False)')
-
-
 class Backup(BASE, CinderBase):
     """Represents a backup of a volume to Swift."""
     __tablename__ = 'backups'
@@ -541,6 +739,9 @@ class Backup(BASE, CinderBase):
     temp_volume_id = Column(String(36))
     temp_snapshot_id = Column(String(36))
     num_dependent_backups = Column(Integer)
+    snapshot_id = Column(String(36))
+    data_timestamp = Column(DateTime)
+    restore_volume_id = Column(String(36))
 
     @validates('fail_reason')
     def validate_fail_reason(self, key, fail_reason):
@@ -601,6 +802,23 @@ class DriverInitiatorData(BASE, models.TimestampMixin, models.ModelBase):
     value = Column(String(255))
 
 
+class Message(BASE, CinderBase):
+    """Represents a message"""
+    __tablename__ = 'messages'
+    id = Column(String(36), primary_key=True, nullable=False)
+    project_id = Column(String(36), nullable=False)
+    # Info/Error/Warning.
+    message_level = Column(String(255), nullable=False)
+    request_id = Column(String(255), nullable=True)
+    resource_type = Column(String(255))
+    # The uuid of the related resource.
+    resource_uuid = Column(String(36), nullable=True)
+    # Operation specific event ID.
+    event_id = Column(String(255), nullable=False)
+    # After this time the message may no longer exist
+    expires_at = Column(DateTime, nullable=True)
+
+
 class ImageVolumeCacheEntry(BASE, models.ModelBase):
     """Represents an image volume cache entry"""
     __tablename__ = 'image_volume_cache_entries'
@@ -613,28 +831,38 @@ class ImageVolumeCacheEntry(BASE, models.ModelBase):
     last_used = Column(DateTime, default=lambda: timeutils.utcnow())
 
 
-def register_models():
-    """Register Models and create metadata.
+class Worker(BASE, CinderBase):
+    """Represents all resources that are being worked on by a node."""
+    __tablename__ = 'workers'
+    __table_args__ = (schema.UniqueConstraint('resource_type', 'resource_id'),
+                      {'mysql_engine': 'InnoDB'})
 
-    Called from cinder.db.sqlalchemy.__init__ as part of loading the driver,
-    it will never need to be called explicitly elsewhere unless the
-    connection is lost and needs to be reestablished.
-    """
-    from sqlalchemy import create_engine
-    models = (Backup,
-              Service,
-              Volume,
-              VolumeMetadata,
-              VolumeAdminMetadata,
-              VolumeAttachment,
-              SnapshotMetadata,
-              Transfer,
-              VolumeTypeExtraSpecs,
-              VolumeTypes,
-              VolumeGlanceMetadata,
-              ConsistencyGroup,
-              Cgsnapshot
-              )
-    engine = create_engine(CONF.database.connection, echo=False)
-    for model in models:
-        model.metadata.create_all(engine)
+    # We want to overwrite default updated_at definition so we timestamp at
+    # creation as well
+    updated_at = Column(DateTime, default=timeutils.utcnow,
+                        onupdate=timeutils.utcnow)
+
+    # Id added for convenience and speed on some operations
+    id = Column(Integer, primary_key=True)
+
+    # Type of the resource we are working on (Volume, Snapshot, Backup) it must
+    # match the Versioned Object class name.
+    resource_type = Column(String(40), primary_key=True, nullable=False)
+    # UUID of the resource we are working on
+    resource_id = Column(String(36), primary_key=True, nullable=False)
+
+    # Status that should be cleaned on service failure
+    status = Column(String(255), nullable=False)
+
+    # Service that is currently processing the operation
+    service_id = Column(Integer, nullable=True)
+
+    # This is a flag we don't need to store in the DB as it is only used when
+    # we are doing the cleanup to let decorators know
+    cleaning = False
+
+    service = relationship(
+        'Service',
+        backref="workers",
+        foreign_keys=service_id,
+        primaryjoin='Worker.service_id == Service.id')
