@@ -12,24 +12,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
+from oslo_utils import versionutils
 from oslo_versionedobjects import fields
 
+from cinder import db
 from cinder import exception
 from cinder.i18n import _
 from cinder import objects
 from cinder.objects import base
 from cinder.volume import volume_types
 
-OPTIONAL_FIELDS = ['extra_specs', 'projects']
-LOG = logging.getLogger(__name__)
-
 
 @base.CinderObjectRegistry.register
 class VolumeType(base.CinderPersistentObject, base.CinderObject,
                  base.CinderObjectDictCompat, base.CinderComparableObject):
     # Version 1.0: Initial version
-    VERSION = '1.0'
+    # Version 1.1: Changed extra_specs to DictOfNullableStringsField
+    # Version 1.2: Added qos_specs
+    VERSION = '1.2'
+
+    OPTIONAL_FIELDS = ('extra_specs', 'projects', 'qos_specs')
 
     fields = {
         'id': fields.UUIDField(),
@@ -37,19 +39,34 @@ class VolumeType(base.CinderPersistentObject, base.CinderObject,
         'description': fields.StringField(nullable=True),
         'is_public': fields.BooleanField(default=True, nullable=True),
         'projects': fields.ListOfStringsField(nullable=True),
-        'extra_specs': fields.DictOfStringsField(nullable=True),
+        'extra_specs': fields.DictOfNullableStringsField(nullable=True),
+        'qos_specs': fields.ObjectField('QualityOfServiceSpecs',
+                                        nullable=True),
     }
 
+    def obj_make_compatible(self, primitive, target_version):
+        super(VolumeType, self).obj_make_compatible(primitive, target_version)
+
+        target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 1):
+            if primitive.get('extra_specs'):
+                # Before 1.1 extra_specs field didn't allowed None values. To
+                # make sure we won't explode on receiver side - change Nones to
+                # empty string.
+                for k, v in primitive['extra_specs'].items():
+                    if v is None:
+                        primitive['extra_specs'][k] = ''
+
     @classmethod
-    def _get_expected_attrs(cls, context):
+    def _get_expected_attrs(cls, context, *args, **kwargs):
         return 'extra_specs', 'projects'
 
-    @staticmethod
-    def _from_db_object(context, type, db_type, expected_attrs=None):
+    @classmethod
+    def _from_db_object(cls, context, type, db_type, expected_attrs=None):
         if expected_attrs is None:
-            expected_attrs = []
+            expected_attrs = ['extra_specs', 'projects']
         for name, field in type.fields.items():
-            if name in OPTIONAL_FIELDS:
+            if name in cls.OPTIONAL_FIELDS:
                 continue
             value = db_type[name]
             if isinstance(field, fields.IntegerField):
@@ -68,12 +85,14 @@ class VolumeType(base.CinderPersistentObject, base.CinderObject,
                 type.extra_specs = specs
         if 'projects' in expected_attrs:
             type.projects = db_type.get('projects', [])
-
+        if 'qos_specs' in expected_attrs:
+            qos_specs = objects.QualityOfServiceSpecs(context)
+            qos_specs._from_db_object(context, qos_specs, db_type['qos_specs'])
+            type.qos_specs = qos_specs
         type._context = context
         type.obj_reset_changes()
         return type
 
-    @base.remotable
     def create(self):
         if self.obj_attr_is_set('id'):
             raise exception.ObjectActionError(action='create',
@@ -84,7 +103,6 @@ class VolumeType(base.CinderPersistentObject, base.CinderObject,
                                              self.description)
         self._from_db_object(self._context, self, db_volume_type)
 
-    @base.remotable
     def save(self):
         updates = self.cinder_obj_get_changes()
         if updates:
@@ -92,10 +110,11 @@ class VolumeType(base.CinderPersistentObject, base.CinderObject,
                                 self.description)
             self.obj_reset_changes()
 
-    @base.remotable
     def destroy(self):
         with self.obj_as_admin():
-            volume_types.destroy(self._context, self.id)
+            updated_values = volume_types.destroy(self._context, self.id)
+        self.update(updated_values)
+        self.obj_reset_changes(updated_values.keys())
 
 
 @base.CinderObjectRegistry.register
@@ -108,12 +127,7 @@ class VolumeTypeList(base.ObjectListBase, base.CinderObject):
         'objects': fields.ListOfObjectsField('VolumeType'),
     }
 
-    child_versions = {
-        '1.0': '1.0',
-        '1.1': '1.0',
-    }
-
-    @base.remotable_classmethod
+    @classmethod
     def get_all(cls, context, inactive=0, filters=None, marker=None,
                 limit=None, sort_keys=None, sort_dirs=None, offset=None):
         types = volume_types.get_all_types(context, inactive, filters,
@@ -124,3 +138,19 @@ class VolumeTypeList(base.ObjectListBase, base.CinderObject):
         return base.obj_make_list(context, cls(context),
                                   objects.VolumeType, types.values(),
                                   expected_attrs=expected_attrs)
+
+    @classmethod
+    def get_all_types_for_qos(cls, context, qos_id):
+        types = db.qos_specs_associations_get(context, qos_id)
+        return base.obj_make_list(context, cls(context), objects.VolumeType,
+                                  types)
+
+    @classmethod
+    def get_all_by_group(cls, context, group_id):
+        # Generic volume group
+        types = volume_types.get_all_types_by_group(
+            context.elevated(), group_id)
+        expected_attrs = VolumeType._get_expected_attrs(context)
+        return base.obj_make_list(context, cls(context),
+                                  objects.VolumeType, types,
+                                  expected_attrs=expected_attrs)
