@@ -41,9 +41,8 @@ from cinder.db import base as cinder_db_base
 from cinder import exception as cinder_exception
 from cinder.image import glance as cinder_image_glance
 from cinder.image import image_utils as cinder_image_imageutils
-import cinder.keymgr
 from cinder.keymgr import conf_key_mgr as cinder_keymgr_confkeymgr
-from cinder.keymgr import key_mgr as cinder_keymgr_keymgr
+from cinder.message import api as cinder_message_api
 from cinder import quota as cinder_quota
 from cinder.scheduler import driver as cinder_scheduler_driver
 from cinder.scheduler import host_manager as cinder_scheduler_hostmanager
@@ -66,6 +65,10 @@ from cinder.volume.drivers import blockbridge as \
 from cinder.volume.drivers.cloudbyte import options as \
     cinder_volume_drivers_cloudbyte_options
 from cinder.volume.drivers import coho as cinder_volume_drivers_coho
+from cinder.volume.drivers.coprhd import common as \
+    cinder_volume_drivers_coprhd_common
+from cinder.volume.drivers.coprhd import scaleio as \
+    cinder_volume_drivers_coprhd_scaleio
 from cinder.volume.drivers import datera as cinder_volume_drivers_datera
 from cinder.volume.drivers.dell import dell_storagecenter_common as \
     cinder_volume_drivers_dell_dellstoragecentercommon
@@ -77,15 +80,19 @@ from cinder.volume.drivers import drbdmanagedrv as \
     cinder_volume_drivers_drbdmanagedrv
 from cinder.volume.drivers.emc import emc_vmax_common as \
     cinder_volume_drivers_emc_emcvmaxcommon
-from cinder.volume.drivers.emc import emc_vnx_cli as \
-    cinder_volume_drivers_emc_emcvnxcli
 from cinder.volume.drivers.emc import scaleio as \
     cinder_volume_drivers_emc_scaleio
+from cinder.volume.drivers.emc.vnx import common as \
+    cinder_volume_drivers_emc_vnx_common
 from cinder.volume.drivers.emc import xtremio as \
     cinder_volume_drivers_emc_xtremio
 from cinder.volume.drivers import eqlx as cinder_volume_drivers_eqlx
+from cinder.volume.drivers.falconstor import fss_common as \
+    cinder_volume_drivers_falconstor_fsscommon
 from cinder.volume.drivers.fujitsu import eternus_dx_common as \
     cinder_volume_drivers_fujitsu_eternusdxcommon
+from cinder.volume.drivers.fusionstorage import dsware as \
+    cinder_volume_drivers_fusionstorage_dsware
 from cinder.volume.drivers import glusterfs as cinder_volume_drivers_glusterfs
 from cinder.volume.drivers import hgst as cinder_volume_drivers_hgst
 from cinder.volume.drivers.hitachi import hbsd_common as \
@@ -100,6 +107,8 @@ from cinder.volume.drivers.hitachi import hnas_iscsi as \
     cinder_volume_drivers_hitachi_hnasiscsi
 from cinder.volume.drivers.hitachi import hnas_nfs as \
     cinder_volume_drivers_hitachi_hnasnfs
+from cinder.volume.drivers.hitachi import hnas_utils as \
+    cinder_volume_drivers_hitachi_hnasutils
 from cinder.volume.drivers.hpe import hpe_3par_common as \
     cinder_volume_drivers_hpe_hpe3parcommon
 from cinder.volume.drivers.hpe import hpe_lefthand_iscsi as \
@@ -115,16 +124,18 @@ from cinder.volume.drivers.ibm import flashsystem_fc as \
 from cinder.volume.drivers.ibm import flashsystem_iscsi as \
     cinder_volume_drivers_ibm_flashsystemiscsi
 from cinder.volume.drivers.ibm import gpfs as cinder_volume_drivers_ibm_gpfs
+from cinder.volume.drivers.ibm import ibm_storage as \
+    cinder_volume_drivers_ibm_ibmstorage
 from cinder.volume.drivers.ibm.storwize_svc import storwize_svc_common as \
     cinder_volume_drivers_ibm_storwize_svc_storwizesvccommon
 from cinder.volume.drivers.ibm.storwize_svc import storwize_svc_fc as \
     cinder_volume_drivers_ibm_storwize_svc_storwizesvcfc
 from cinder.volume.drivers.ibm.storwize_svc import storwize_svc_iscsi as \
     cinder_volume_drivers_ibm_storwize_svc_storwizesvciscsi
-from cinder.volume.drivers.ibm import xiv_ds8k as \
-    cinder_volume_drivers_ibm_xivds8k
 from cinder.volume.drivers.infortrend.eonstor_ds_cli import common_cli as \
     cinder_volume_drivers_infortrend_eonstor_ds_cli_commoncli
+from cinder.volume.drivers.kaminario import kaminario_common as \
+    cinder_volume_drivers_kaminario_kaminariocommon
 from cinder.volume.drivers.lenovo import lenovo_common as \
     cinder_volume_drivers_lenovo_lenovocommon
 from cinder.volume.drivers import lvm as cinder_volume_drivers_lvm
@@ -147,6 +158,8 @@ from cinder.volume.drivers import scality as cinder_volume_drivers_scality
 from cinder.volume.drivers import sheepdog as cinder_volume_drivers_sheepdog
 from cinder.volume.drivers import smbfs as cinder_volume_drivers_smbfs
 from cinder.volume.drivers import solidfire as cinder_volume_drivers_solidfire
+from cinder.volume.drivers.synology import synology_common as \
+    cinder_volume_drivers_synology_synologycommon
 from cinder.volume.drivers import tegile as cinder_volume_drivers_tegile
 from cinder.volume.drivers import tintri as cinder_volume_drivers_tintri
 from cinder.volume.drivers.violin import v7000_common as \
@@ -157,10 +170,12 @@ from cinder.volume.drivers import vzstorage as cinder_volume_drivers_vzstorage
 from cinder.volume.drivers.windows import windows as \
     cinder_volume_drivers_windows_windows
 from cinder.volume.drivers import xio as cinder_volume_drivers_xio
+from cinder.volume.drivers import zadara as cinder_volume_drivers_zadara
 from cinder.volume.drivers.zfssa import zfssaiscsi as \
     cinder_volume_drivers_zfssa_zfssaiscsi
 from cinder.volume.drivers.zfssa import zfssanfs as \
     cinder_volume_drivers_zfssa_zfssanfs
+from cinder.volume.drivers.zte import zte_ks as cinder_volume_drivers_zte_zteks
 from cinder.volume import manager as cinder_volume_manager
 from cinder.wsgi import eventlet_server as cinder_wsgi_eventletserver
 from cinder.zonemanager.drivers.brocade import brcd_fabric_opts as \
@@ -179,19 +194,15 @@ def list_opts():
     return [
         ('FC-ZONE-MANAGER',
             itertools.chain(
-                cinder_zonemanager_fczonemanager.zone_manager_opts,
                 cinder_zonemanager_drivers_brocade_brcdfczonedriver.brcd_opts,
+                cinder_zonemanager_fczonemanager.zone_manager_opts,
                 cinder_zonemanager_drivers_cisco_ciscofczonedriver.cisco_opts,
             )),
-        ('KEYMGR',
-            itertools.chain(
-                cinder_keymgr_keymgr.encryption_opts,
-                cinder.keymgr.keymgr_opts,
-                cinder_keymgr_confkeymgr.key_mgr_opts,
-            )),
         ('DEFAULT',
             itertools.chain(
                 cinder_backup_driver.service_opts,
+                [cinder_cmd_volume.cluster_opt],
+                cinder_volume_drivers_hitachi_hnasutils.drivers_common_opts,
                 cinder_api_common.api_common_opts,
                 cinder_backup_drivers_ceph.service_opts,
                 cinder_volume_drivers_smbfs.volume_opts,
@@ -211,13 +222,16 @@ def list_opts():
                 cinder_volume_drivers_netapp_options.netapp_eseries_opts,
                 cinder_volume_drivers_netapp_options.netapp_nfs_extra_opts,
                 cinder_volume_drivers_netapp_options.netapp_san_opts,
+                cinder_volume_drivers_netapp_options.netapp_replication_opts,
                 cinder_volume_drivers_ibm_storwize_svc_storwizesvciscsi.
                 storwize_svc_iscsi_opts,
                 cinder_backup_drivers_glusterfs.glusterfsbackup_service_opts,
+                cinder_volume_drivers_coprhd_scaleio.scaleio_opts,
                 cinder_backup_drivers_tsm.tsm_opts,
                 cinder_volume_drivers_fujitsu_eternusdxcommon.
                 FJ_ETERNUS_DX_OPT_opts,
                 cinder_volume_drivers_ibm_gpfs.gpfs_opts,
+                cinder_volume_drivers_zte_zteks.zte_opts,
                 cinder_volume_drivers_violin_v7000common.violin_opts,
                 cinder_volume_drivers_nexenta_options.NEXENTA_CONNECTION_OPTS,
                 cinder_volume_drivers_nexenta_options.NEXENTA_ISCSI_OPTS,
@@ -226,11 +240,13 @@ def list_opts():
                 cinder_volume_drivers_nexenta_options.NEXENTA_RRMGR_OPTS,
                 cinder_volume_drivers_nexenta_options.NEXENTA_EDGE_OPTS,
                 cinder_exception.exc_log_opts,
+                cinder_common_config.core_opts,
                 cinder_common_config.global_opts,
                 cinder_scheduler_weights_capacity.capacity_weight_opts,
                 cinder_volume_drivers_sheepdog.sheepdog_opts,
                 [cinder_api_middleware_sizelimit.max_request_body_size_opt],
                 cinder_volume_drivers_solidfire.sf_opts,
+                cinder_volume_drivers_coprhd_common.volume_opts,
                 cinder_backup_drivers_swift.swiftbackup_service_opts,
                 cinder_volume_drivers_cloudbyte_options.
                 cloudbyte_add_qosgroup_opts,
@@ -249,8 +265,7 @@ def list_opts():
                 cinder_volume_drivers_pure.PURE_OPTS,
                 cinder_context.context_opts,
                 cinder_scheduler_driver.scheduler_driver_opts,
-                cinder_volume_drivers_scality.volume_opts,
-                cinder_volume_drivers_emc_emcvnxcli.loc_opts,
+                cinder_volume_drivers_ibm_ibmstorage.driver_opts,
                 cinder_volume_drivers_vmware_vmdk.vmdk_opts,
                 cinder_volume_drivers_lenovo_lenovocommon.common_opts,
                 cinder_volume_drivers_lenovo_lenovocommon.iscsi_opts,
@@ -263,15 +278,18 @@ def list_opts():
                 cinder_scheduler_weights_volumenumber.
                 volume_number_weight_opts,
                 cinder_volume_drivers_coho.coho_opts,
+                cinder_volume_drivers_scality.volume_opts,
                 cinder_volume_drivers_xio.XIO_OPTS,
                 cinder_volume_drivers_ibm_storwize_svc_storwizesvcfc.
                 storwize_svc_fc_opts,
+                cinder_volume_drivers_falconstor_fsscommon.FSS_OPTS,
                 cinder_volume_drivers_zfssa_zfssaiscsi.ZFSSA_OPTS,
                 cinder_volume_driver.volume_opts,
                 cinder_volume_driver.iser_opts,
                 cinder_api_views_versions.versions_opts,
                 cinder_volume_drivers_nimble.nimble_opts,
                 cinder_volume_drivers_windows_windows.windows_opts,
+                cinder_volume_drivers_emc_vnx_common.EMC_VNX_OPTS,
                 cinder_volume_drivers_san_hp_hpmsacommon.common_opts,
                 cinder_volume_drivers_san_hp_hpmsacommon.iscsi_opts,
                 cinder_image_glance.glance_opts,
@@ -290,7 +308,7 @@ def list_opts():
                 cinder_volume_drivers_infortrend_eonstor_ds_cli_commoncli.
                 infortrend_esds_extra_opts,
                 cinder_volume_drivers_hitachi_hnasiscsi.iSCSI_OPTS,
-                cinder_volume_drivers_rbd.rbd_opts,
+                cinder_volume_drivers_rbd.RBD_OPTS,
                 cinder_volume_drivers_tintri.tintri_opts,
                 cinder_backup_api.backup_api_opts,
                 cinder_volume_drivers_hitachi_hbsdhorcm.volume_opts,
@@ -300,6 +318,7 @@ def list_opts():
                 cinder_volume_drivers_hitachi_hbsdfc.volume_opts,
                 cinder_quota.quota_opts,
                 cinder_volume_drivers_huawei_huaweidriver.huawei_opts,
+                cinder_volume_drivers_synology_synologycommon.cinder_opts,
                 cinder_volume_drivers_dell_dellstoragecentercommon.
                 common_opts,
                 cinder_scheduler_hostmanager.host_manager_opts,
@@ -309,8 +328,12 @@ def list_opts():
                 [cinder_scheduler_scheduleroptions.
                     scheduler_json_config_location_opt],
                 cinder_volume_drivers_zfssa_zfssanfs.ZFSSA_OPTS,
+                cinder_volume_drivers_fusionstorage_dsware.volume_opts,
+                cinder_volume_drivers_kaminario_kaminariocommon.
+                kaminario1_opts,
                 cinder_volume_drivers_disco_disco.disco_opts,
                 cinder_volume_drivers_hgst.hgst_opts,
+                cinder_message_api.messages_opts,
                 cinder_image_imageutils.image_helper_opts,
                 cinder_compute_nova.nova_opts,
                 cinder_volume_drivers_ibm_flashsystemfc.flashsystem_fc_opts,
@@ -328,9 +351,9 @@ def list_opts():
                 [cinder_volume_api.volume_host_opt],
                 [cinder_volume_api.volume_same_az_opt],
                 [cinder_volume_api.az_cache_time_opt],
-                cinder_volume_drivers_ibm_xivds8k.xiv_ds8k_opts,
                 cinder_volume_drivers_hpe_hpe3parcommon.hpe3par_opts,
                 cinder_volume_drivers_datera.d_opts,
+                cinder_volume_drivers_zadara.zadara_opts,
                 cinder_volume_drivers_blockdevice.volume_opts,
                 cinder_volume_drivers_quobyte.volume_opts,
                 cinder_volume_drivers_vzstorage.vzstorage_opts,
@@ -350,6 +373,10 @@ def list_opts():
             itertools.chain(
                 cinder_coordination.coordination_opts,
             )),
+        ('KEY_MANAGER',
+            itertools.chain(
+                cinder_keymgr_confkeymgr.key_mgr_opts,
+            )),
         ('BACKEND',
             itertools.chain(
                 [cinder_cmd_volume.host_opt],
