@@ -13,10 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from lxml import etree
 import mock
 from oslo_config import cfg
-from oslo_utils import timeutils
 from six.moves.urllib import parse as urllib
 import webob
 
@@ -26,9 +24,11 @@ from cinder import context
 from cinder import db
 from cinder import exception
 from cinder import objects
+from cinder.objects import fields
 from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import stubs
+from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit import utils
@@ -44,10 +44,11 @@ INVALID_UUID = '00000000-0000-0000-0000-000000000002'
 def _get_default_snapshot_param():
     return {
         'id': UUID,
-        'volume_id': 12,
-        'status': 'available',
+        'volume_id': fake.VOLUME_ID,
+        'status': fields.SnapshotStatus.AVAILABLE,
         'volume_size': 100,
         'created_at': None,
+        'updated_at': None,
         'user_id': 'bcb7746c7a41472d88a1ffac89ba6a9b',
         'project_id': '7ffe17a15c724e2aa79fc839540aec15',
         'display_name': 'Default name',
@@ -85,7 +86,7 @@ class SnapshotApiTest(test.TestCase):
         self.stubs.Set(db, 'snapshot_get_all',
                        stubs.stub_snapshot_get_all)
 
-        self.ctx = context.RequestContext('admin', 'fakeproject', True)
+        self.ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, True)
 
     @mock.patch(
         'cinder.api.openstack.wsgi.Controller.validate_name_and_description')
@@ -109,6 +110,7 @@ class SnapshotApiTest(test.TestCase):
         self.assertEqual(snapshot_description,
                          resp_dict['snapshot']['description'])
         self.assertTrue(mock_validate.called)
+        self.assertIn('updated_at', resp_dict['snapshot'])
         db.volume_destroy(self.ctx, volume.id)
 
     def test_snapshot_create_force(self):
@@ -130,6 +132,7 @@ class SnapshotApiTest(test.TestCase):
                          resp_dict['snapshot']['name'])
         self.assertEqual(snapshot_description,
                          resp_dict['snapshot']['description'])
+        self.assertIn('updated_at', resp_dict['snapshot'])
 
         snapshot = {
             "volume_id": volume.id,
@@ -172,14 +175,14 @@ class SnapshotApiTest(test.TestCase):
             snapshot_metadata_get, update_snapshot):
         snapshot = {
             'id': UUID,
-            'volume_id': 1,
-            'status': 'available',
+            'volume_id': fake.VOLUME_ID,
+            'status': fields.SnapshotStatus.AVAILABLE,
             'volume_size': 100,
             'display_name': 'Default name',
             'display_description': 'Default description',
             'expected_attrs': ['metadata'],
         }
-        ctx = context.RequestContext('admin', 'fake', True)
+        ctx = context.RequestContext(fake.PROJECT_ID, fake.USER_ID, True)
         snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
         fake_volume_obj = fake_volume.fake_volume_obj(ctx)
         snapshot_get_by_id.return_value = snapshot_obj
@@ -194,10 +197,11 @@ class SnapshotApiTest(test.TestCase):
         expected = {
             'snapshot': {
                 'id': UUID,
-                'volume_id': '1',
-                'status': u'available',
+                'volume_id': fake.VOLUME_ID,
+                'status': fields.SnapshotStatus.AVAILABLE,
                 'size': 100,
                 'created_at': None,
+                'updated_at': None,
                 'name': u'Updated Test Name',
                 'description': u'Default description',
                 'metadata': {},
@@ -226,8 +230,8 @@ class SnapshotApiTest(test.TestCase):
         }
         body = {"snapshot": updates}
         req = fakes.HTTPRequest.blank('/v2/snapshots/not-the-uuid')
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.update, req,
-                          'not-the-uuid', body)
+        self.assertRaises(exception.SnapshotNotFound, self.controller.update,
+                          req, 'not-the-uuid', body)
 
     @mock.patch.object(volume.api.API, "delete_snapshot",
                        side_effect=stubs.stub_snapshot_update)
@@ -238,14 +242,14 @@ class SnapshotApiTest(test.TestCase):
                              snapshot_metadata_get, delete_snapshot):
         snapshot = {
             'id': UUID,
-            'volume_id': 1,
-            'status': 'available',
+            'volume_id': fake.VOLUME_ID,
+            'status': fields.SnapshotStatus.AVAILABLE,
             'volume_size': 100,
             'display_name': 'Default name',
             'display_description': 'Default description',
             'expected_attrs': ['metadata'],
         }
-        ctx = context.RequestContext('admin', 'fake', True)
+        ctx = context.RequestContext(fake.PROJECT_ID, fake.USER_ID, True)
         snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
         fake_volume_obj = fake_volume.fake_volume_obj(ctx)
         snapshot_get_by_id.return_value = snapshot_obj
@@ -260,7 +264,7 @@ class SnapshotApiTest(test.TestCase):
         self.stubs.Set(volume.api.API, "delete_snapshot", stub_snapshot_delete)
         snapshot_id = INVALID_UUID
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % snapshot_id)
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
+        self.assertRaises(exception.SnapshotNotFound, self.controller.delete,
                           req, snapshot_id)
 
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
@@ -270,14 +274,14 @@ class SnapshotApiTest(test.TestCase):
                            snapshot_metadata_get):
         snapshot = {
             'id': UUID,
-            'volume_id': 1,
-            'status': 'available',
+            'volume_id': fake.VOLUME_ID,
+            'status': fields.SnapshotStatus.AVAILABLE,
             'volume_size': 100,
             'display_name': 'Default name',
             'display_description': 'Default description',
             'expected_attrs': ['metadata'],
         }
-        ctx = context.RequestContext('admin', 'fake', True)
+        ctx = context.RequestContext(fake.PROJECT_ID, fake.USER_ID, True)
         snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
         fake_volume_obj = fake_volume.fake_volume_obj(ctx)
         snapshot_get_by_id.return_value = snapshot_obj
@@ -287,11 +291,12 @@ class SnapshotApiTest(test.TestCase):
 
         self.assertIn('snapshot', resp_dict)
         self.assertEqual(UUID, resp_dict['snapshot']['id'])
+        self.assertIn('updated_at', resp_dict['snapshot'])
 
     def test_snapshot_show_invalid_id(self):
         snapshot_id = INVALID_UUID
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % snapshot_id)
-        self.assertRaises(webob.exc.HTTPNotFound,
+        self.assertRaises(exception.SnapshotNotFound,
                           self.controller.show, req, snapshot_id)
 
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
@@ -302,14 +307,14 @@ class SnapshotApiTest(test.TestCase):
                              volume_get_by_id, snapshot_metadata_get):
         snapshot = {
             'id': UUID,
-            'volume_id': 1,
-            'status': 'available',
+            'volume_id': fake.VOLUME_ID,
+            'status': fields.SnapshotStatus.AVAILABLE,
             'volume_size': 100,
             'display_name': 'Default name',
             'display_description': 'Default description',
             'expected_attrs': ['metadata']
         }
-        ctx = context.RequestContext('admin', 'fake', True)
+        ctx = context.RequestContext(fake.PROJECT_ID, fake.USER_ID, True)
         snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
         fake_volume_obj = fake_volume.fake_volume_obj(ctx)
         snapshot_get_by_id.return_value = snapshot_obj
@@ -323,6 +328,7 @@ class SnapshotApiTest(test.TestCase):
         self.assertIn('snapshots', resp_dict)
         resp_snapshots = resp_dict['snapshots']
         self.assertEqual(1, len(resp_snapshots))
+        self.assertIn('updated_at', resp_snapshots[0])
 
         resp_snapshot = resp_snapshots.pop()
         self.assertEqual(UUID, resp_snapshot['id'])
@@ -330,7 +336,7 @@ class SnapshotApiTest(test.TestCase):
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_admin_list_snapshots_limited_to_project(self,
                                                      snapshot_metadata_get):
-        req = fakes.HTTPRequest.blank('/v2/fake/snapshots',
+        req = fakes.HTTPRequest.blank('/v2/%s/snapshots' % fake.PROJECT_ID,
                                       use_admin_context=True)
         res = self.controller.index(req)
 
@@ -341,14 +347,15 @@ class SnapshotApiTest(test.TestCase):
     def test_list_snapshots_with_limit_and_offset(self,
                                                   snapshot_metadata_get):
         def list_snapshots_with_limit_and_offset(snaps, is_admin):
-            req = fakes.HTTPRequest.blank('/v2/fake/snapshots?limit=1\
-                                          &offset=1',
+            req = fakes.HTTPRequest.blank('/v2/%s/snapshots?limit=1'
+                                          '&offset=1' % fake.PROJECT_ID,
                                           use_admin_context=is_admin)
             res = self.controller.index(req)
 
             self.assertIn('snapshots', res)
             self.assertEqual(1, len(res['snapshots']))
             self.assertEqual(snaps[1].id, res['snapshots'][0]['id'])
+            self.assertIn('updated_at', res['snapshots'][0])
 
             # Test that we get an empty list with an offset greater than the
             # number of items
@@ -394,13 +401,20 @@ class SnapshotApiTest(test.TestCase):
                           self.controller.index,
                           req)
 
-    def _assert_list_next(self, expected_query=None, project='fakeproject',
+        # Test that we get an exception HTTPBadRequest(400) with an offset
+        # greater than the maximum offset value.
+        url = '/v2/snapshots?limit=1&offset=323245324356534235'
+        req = fakes.HTTPRequest.blank(url)
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.index, req)
+
+    def _assert_list_next(self, expected_query=None, project=fake.PROJECT_ID,
                           **kwargs):
         """Check a page of snapshots list."""
         # Since we are accessing v2 api directly we don't need to specify
         # v2 in the request path, if we did, we'd get /v2/v2 links back
-        request_path = '/%s/snapshots' % project
-        expected_path = '/v2' + request_path
+        request_path = '/v2/%s/snapshots' % project
+        expected_path = request_path
 
         # Construct the query if there are kwargs
         if kwargs:
@@ -506,29 +520,52 @@ class SnapshotApiTest(test.TestCase):
 
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_admin_list_snapshots_all_tenants(self, snapshot_metadata_get):
-        req = fakes.HTTPRequest.blank('/v2/fake/snapshots?all_tenants=1',
+        req = fakes.HTTPRequest.blank('/v2/%s/snapshots?all_tenants=1' %
+                                      fake.PROJECT_ID,
                                       use_admin_context=True)
         res = self.controller.index(req)
         self.assertIn('snapshots', res)
         self.assertEqual(3, len(res['snapshots']))
 
+    @mock.patch.object(db, 'snapshot_get_all')
+    @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
+    def test_admin_list_snapshots_by_tenant_id(self, snapshot_metadata_get,
+                                               snapshot_get_all):
+        def get_all(context, filters=None, marker=None, limit=None,
+                    sort_keys=None, sort_dirs=None, offset=None):
+            if 'project_id' in filters and 'tenant1' in filters['project_id']:
+                return [stubs.stub_snapshot(fake.VOLUME_ID,
+                                            tenant_id='tenant1')]
+            else:
+                return []
+
+        snapshot_get_all.side_effect = get_all
+
+        req = fakes.HTTPRequest.blank('/v2/%s/snapshots?all_tenants=1'
+                                      '&project_id=tenant1' % fake.PROJECT_ID,
+                                      use_admin_context=True)
+        res = self.controller.index(req)
+        self.assertIn('snapshots', res)
+        self.assertEqual(1, len(res['snapshots']))
+
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_all_tenants_non_admin_gets_all_tenants(self,
                                                     snapshot_metadata_get):
-        req = fakes.HTTPRequest.blank('/v2/fake/snapshots?all_tenants=1')
+        req = fakes.HTTPRequest.blank('/v2/%s/snapshots?all_tenants=1' %
+                                      fake.PROJECT_ID)
         res = self.controller.index(req)
         self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_non_admin_get_by_project(self, snapshot_metadata_get):
-        req = fakes.HTTPRequest.blank('/v2/fake/snapshots')
+        req = fakes.HTTPRequest.blank('/v2/%s/snapshots' % fake.PROJECT_ID)
         res = self.controller.index(req)
         self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
     def _create_snapshot_bad_body(self, body):
-        req = fakes.HTTPRequest.blank('/v2/fake/snapshots')
+        req = fakes.HTTPRequest.blank('/v2/%s/snapshots' % fake.PROJECT_ID)
         req.method = 'POST'
 
         self.assertRaises(webob.exc.HTTPBadRequest,
@@ -544,61 +581,3 @@ class SnapshotApiTest(test.TestCase):
     def test_create_malformed_entity(self):
         body = {'snapshot': 'string'}
         self._create_snapshot_bad_body(body=body)
-
-
-class SnapshotSerializerTest(test.TestCase):
-    def _verify_snapshot(self, snap, tree):
-        self.assertEqual('snapshot', tree.tag)
-
-        for attr in ('id', 'status', 'size', 'created_at',
-                     'name', 'description', 'volume_id'):
-            self.assertEqual(str(snap[attr]), tree.get(attr))
-
-    def test_snapshot_show_create_serializer(self):
-        serializer = snapshots.SnapshotTemplate()
-        raw_snapshot = dict(
-            id='snap_id',
-            status='snap_status',
-            size=1024,
-            created_at=timeutils.utcnow(),
-            name='snap_name',
-            description='snap_desc',
-            display_description='snap_desc',
-            volume_id='vol_id',
-        )
-        text = serializer.serialize(dict(snapshot=raw_snapshot))
-
-        tree = etree.fromstring(text)
-
-        self._verify_snapshot(raw_snapshot, tree)
-
-    def test_snapshot_index_detail_serializer(self):
-        serializer = snapshots.SnapshotsTemplate()
-        raw_snapshots = [
-            dict(
-                id='snap1_id',
-                status='snap1_status',
-                size=1024,
-                created_at=timeutils.utcnow(),
-                name='snap1_name',
-                description='snap1_desc',
-                volume_id='vol1_id',
-            ),
-            dict(
-                id='snap2_id',
-                status='snap2_status',
-                size=1024,
-                created_at=timeutils.utcnow(),
-                name='snap2_name',
-                description='snap2_desc',
-                volume_id='vol2_id',
-            )
-        ]
-        text = serializer.serialize(dict(snapshots=raw_snapshots))
-
-        tree = etree.fromstring(text)
-
-        self.assertEqual('snapshots', tree.tag)
-        self.assertEqual(len(raw_snapshots), len(tree))
-        for idx, child in enumerate(tree):
-            self._verify_snapshot(raw_snapshots[idx], child)
