@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import mock
-from mox3 import mox
 from oslo_concurrency import processutils
 
 from cinder.brick.local_dev import lvm as brick
@@ -22,26 +21,22 @@ from cinder import test
 from cinder.volume import configuration as conf
 
 
-def create_configuration():
-    configuration = mox.MockObject(conf.Configuration)
-    configuration.append_config_values(mox.IgnoreArg())
-    return configuration
-
-
 class BrickLvmTestCase(test.TestCase):
     def setUp(self):
-        self.configuration = mox.MockObject(conf.Configuration)
+        if not hasattr(self, 'configuration'):
+            self.configuration = mock.Mock(conf.Configuration)
+            self.configuration.lvm_suppress_fd_warnings = False
         self.configuration.volume_group_name = 'fake-vg'
         super(BrickLvmTestCase, self).setUp()
 
-        # Stub processutils.execute for static methods
-        self.stubs.Set(processutils, 'execute',
-                       self.fake_execute)
-        self.vg = brick.LVM(self.configuration.volume_group_name,
-                            'sudo',
-                            False, None,
-                            'default',
-                            self.fake_execute)
+        self.mock_object(processutils, 'execute', self.fake_execute)
+        self.vg = brick.LVM(
+            self.configuration.volume_group_name,
+            'sudo',
+            False, None,
+            'default',
+            self.fake_execute,
+            suppress_fd_warn=self.configuration.lvm_suppress_fd_warnings)
 
     def failed_fake_execute(obj, *cmd, **kwargs):
         return ("\n", "fake-error")
@@ -56,25 +51,29 @@ class BrickLvmTestCase(test.TestCase):
     def fake_customised_lvm_version(obj, *cmd, **kwargs):
         return ("  LVM version:     2.02.100(2)-RHEL6 (2013-09-12)\n", "")
 
-    def fake_execute(obj, *cmd, **kwargs):
+    def fake_execute(obj, *cmd, **kwargs):  # noqa
+        if obj.configuration.lvm_suppress_fd_warnings:
+            _lvm_prefix = 'env, LC_ALL=C, LVM_SUPPRESS_FD_WARNINGS=1, '
+        else:
+            _lvm_prefix = 'env, LC_ALL=C, '
+
         cmd_string = ', '.join(cmd)
         data = "\n"
-
-        if ('env, LC_ALL=C, vgs, --noheadings, --unit=g, -o, name' ==
+        if (_lvm_prefix + 'vgs, --noheadings, --unit=g, -o, name' ==
                 cmd_string):
             data = "  fake-vg\n"
             data += "  some-other-vg\n"
-        elif ('env, LC_ALL=C, vgs, --noheadings, -o, name, fake-vg' ==
+        elif (_lvm_prefix + 'vgs, --noheadings, -o, name, fake-vg' ==
                 cmd_string):
             data = "  fake-vg\n"
-        elif 'env, LC_ALL=C, vgs, --version' in cmd_string:
+        elif _lvm_prefix + 'vgs, --version' in cmd_string:
             data = "  LVM version:     2.02.95(2) (2012-03-06)\n"
-        elif ('env, LC_ALL=C, vgs, --noheadings, -o, uuid, fake-vg' in
-              cmd_string):
+        elif(_lvm_prefix + 'vgs, --noheadings, -o, uuid, fake-vg' in
+             cmd_string):
             data = "  kVxztV-dKpG-Rz7E-xtKY-jeju-QsYU-SLG6Z1\n"
-        elif 'env, LC_ALL=C, vgs, --noheadings, --unit=g, ' \
-             '-o, name,size,free,lv_count,uuid, ' \
-             '--separator, :, --nosuffix' in cmd_string:
+        elif(_lvm_prefix + 'vgs, --noheadings, --unit=g, '
+             '-o, name,size,free,lv_count,uuid, '
+             '--separator, :, --nosuffix' in cmd_string):
             data = ("  test-prov-cap-vg-unit:10.00:10.00:0:"
                     "mXzbuX-dKpG-Rz7E-xtKY-jeju-QsYU-SLG8Z4\n")
             if 'test-prov-cap-vg-unit' in cmd_string:
@@ -91,17 +90,17 @@ class BrickLvmTestCase(test.TestCase):
                     "lWyauW-dKpG-Rz7E-xtKY-jeju-QsYU-SLG7Z2\n"
             data += "  fake-vg-3:10.00:10.00:0:"\
                     "mXzbuX-dKpG-Rz7E-xtKY-jeju-QsYU-SLG8Z3\n"
-        elif ('env, LC_ALL=C, lvs, --noheadings, '
+        elif (_lvm_prefix + 'lvs, --noheadings, '
               '--unit=g, -o, vg_name,name,size, --nosuffix, '
               'fake-vg/lv-nothere' in cmd_string):
             raise processutils.ProcessExecutionError(
                 stderr="One or more specified logical volume(s) not found.")
-        elif ('env, LC_ALL=C, lvs, --noheadings, '
+        elif (_lvm_prefix + 'lvs, --noheadings, '
               '--unit=g, -o, vg_name,name,size, --nosuffix, '
               'fake-vg/lv-newerror' in cmd_string):
             raise processutils.ProcessExecutionError(
                 stderr="Failed to find logical volume \"fake-vg/lv-newerror\"")
-        elif ('env, LC_ALL=C, lvs, --noheadings, '
+        elif (_lvm_prefix + 'lvs, --noheadings, '
               '--unit=g, -o, vg_name,name,size' in cmd_string):
             if 'fake-unknown' in cmd_string:
                 raise processutils.ProcessExecutionError(
@@ -120,19 +119,29 @@ class BrickLvmTestCase(test.TestCase):
             else:
                 data = "  fake-vg fake-1 1.00g\n"
                 data += "  fake-vg fake-2 1.00g\n"
-        elif ('env, LC_ALL=C, lvdisplay, --noheading, -C, -o, Attr' in
+        elif (_lvm_prefix + 'lvdisplay, --noheading, -C, -o, Attr' in
               cmd_string):
             if 'test-volumes' in cmd_string:
                 data = '  wi-a-'
+            elif 'snapshot' in cmd_string:
+                data = '  swi-a-s--'
+            elif 'open' in cmd_string:
+                data = '  -wi-ao---'
             else:
                 data = '  owi-a-'
-        elif 'env, LC_ALL=C, pvs, --noheadings' in cmd_string:
+        elif (_lvm_prefix + 'lvdisplay, --noheading, -C, -o, Origin' in
+              cmd_string):
+            if 'snapshot' in cmd_string:
+                data = '  fake-volume-1'
+            else:
+                data = '       '
+        elif _lvm_prefix + 'pvs, --noheadings' in cmd_string:
             data = "  fake-vg|/dev/sda|10.00|1.00\n"
             data += "  fake-vg|/dev/sdb|10.00|1.00\n"
             data += "  fake-vg|/dev/sdc|10.00|8.99\n"
             data += "  fake-vg-2|/dev/sdd|10.00|9.99\n"
-        elif 'env, LC_ALL=C, lvs, --noheadings, --unit=g' \
-             ', -o, size,data_percent, --separator, :' in cmd_string:
+        elif _lvm_prefix + 'lvs, --noheadings, --unit=g' \
+                ', -o, size,data_percent, --separator, :' in cmd_string:
             if 'test-prov-cap-pool' in cmd_string:
                 data = "  9.5:20\n"
             else:
@@ -155,23 +164,17 @@ class BrickLvmTestCase(test.TestCase):
     def test_create_lv_snapshot(self):
         self.assertIsNone(self.vg.create_lv_snapshot('snapshot-1', 'fake-1'))
 
-        self.mox.StubOutWithMock(self.vg, 'get_volume')
-        self.vg.get_volume('fake-non-existent').AndReturn(None)
-        self.mox.ReplayAll()
-        try:
-            self.vg.create_lv_snapshot('snapshot-1', 'fake-non-existent')
-        except exception.VolumeDeviceNotFound as e:
-            self.assertEqual('fake-non-existent', e.kwargs['device'])
-        else:
-            self.fail("Exception not raised")
+        with mock.patch.object(self.vg, 'get_volume', return_value=None):
+            try:
+                self.vg.create_lv_snapshot('snapshot-1', 'fake-non-existent')
+            except exception.VolumeDeviceNotFound as e:
+                self.assertEqual('fake-non-existent', e.kwargs['device'])
+            else:
+                self.fail("Exception not raised")
 
     def test_vg_exists(self):
         self.assertTrue(self.vg._vg_exists())
 
-    def test_get_vg_uuid(self):
-        self.assertEqual('kVxztV-dKpG-Rz7E-xtKY-jeju-QsYU-SLG6Z1',
-                         self.vg._get_vg_uuid()[0])
-
     def test_get_all_volumes(self):
         out = self.vg.get_volumes()
 
@@ -227,10 +230,6 @@ class BrickLvmTestCase(test.TestCase):
         pvs = self.vg.get_all_physical_volumes('sudo')
         self.assertEqual(4, len(pvs))
 
-    def test_get_physical_volumes(self):
-        pvs = self.vg.get_physical_volumes()
-        self.assertEqual(3, len(pvs))
-
     def test_get_volume_groups(self):
         self.assertEqual(3, len(self.vg.get_all_volume_groups('sudo')))
         self.assertEqual(1,
@@ -241,28 +240,28 @@ class BrickLvmTestCase(test.TestCase):
         # use the self._executor fake we pass in on init
         # so we need to stub processutils.execute appropriately
 
-        self.stubs.Set(processutils, 'execute', self.fake_execute)
         self.assertTrue(self.vg.supports_thin_provisioning('sudo'))
 
-        self.stubs.Set(processutils, 'execute', self.fake_pretend_lvm_version)
-        self.assertTrue(self.vg.supports_thin_provisioning('sudo'))
+        with mock.patch.object(processutils, 'execute',
+                               self.fake_pretend_lvm_version):
+            self.assertTrue(self.vg.supports_thin_provisioning('sudo'))
 
-        self.stubs.Set(processutils, 'execute', self.fake_old_lvm_version)
-        self.assertFalse(self.vg.supports_thin_provisioning('sudo'))
+        with mock.patch.object(processutils, 'execute',
+                               self.fake_old_lvm_version):
+            self.assertFalse(self.vg.supports_thin_provisioning('sudo'))
 
-        self.stubs.Set(processutils,
-                       'execute',
-                       self.fake_customised_lvm_version)
-        self.assertTrue(self.vg.supports_thin_provisioning('sudo'))
+        with mock.patch.object(processutils, 'execute',
+                               self.fake_customised_lvm_version):
+            self.assertTrue(self.vg.supports_thin_provisioning('sudo'))
 
     def test_snapshot_lv_activate_support(self):
         self.vg._supports_snapshot_lv_activation = None
-        self.stubs.Set(processutils, 'execute', self.fake_execute)
         self.assertTrue(self.vg.supports_snapshot_lv_activation)
 
         self.vg._supports_snapshot_lv_activation = None
-        self.stubs.Set(processutils, 'execute', self.fake_old_lvm_version)
-        self.assertFalse(self.vg.supports_snapshot_lv_activation)
+        with mock.patch.object(processutils, 'execute',
+                               self.fake_old_lvm_version):
+            self.assertFalse(self.vg.supports_snapshot_lv_activation)
 
         self.vg._supports_snapshot_lv_activation = None
 
@@ -270,12 +269,14 @@ class BrickLvmTestCase(test.TestCase):
         """Tests if lvchange -K is available via a lvm2 version check."""
 
         self.vg._supports_lvchange_ignoreskipactivation = None
-        self.stubs.Set(processutils, 'execute', self.fake_pretend_lvm_version)
-        self.assertTrue(self.vg.supports_lvchange_ignoreskipactivation)
+        with mock.patch.object(processutils, 'execute',
+                               self.fake_pretend_lvm_version):
+            self.assertTrue(self.vg.supports_lvchange_ignoreskipactivation)
 
         self.vg._supports_lvchange_ignoreskipactivation = None
-        self.stubs.Set(processutils, 'execute', self.fake_old_lvm_version)
-        self.assertFalse(self.vg.supports_lvchange_ignoreskipactivation)
+        with mock.patch.object(processutils, 'execute',
+                               self.fake_old_lvm_version):
+            self.assertFalse(self.vg.supports_lvchange_ignoreskipactivation)
 
         self.vg._supports_lvchange_ignoreskipactivation = None
 
@@ -339,19 +340,28 @@ class BrickLvmTestCase(test.TestCase):
         self.assertTrue(self.vg.lv_has_snapshot('fake-vg'))
         self.assertFalse(self.vg.lv_has_snapshot('test-volumes'))
 
+    def test_lv_is_snapshot(self):
+        self.assertTrue(self.vg.lv_is_snapshot('fake-snapshot'))
+        self.assertFalse(self.vg.lv_is_snapshot('test-volumes'))
+
+    def test_lv_is_open(self):
+        self.assertTrue(self.vg.lv_is_open('fake-open'))
+        self.assertFalse(self.vg.lv_is_open('fake-snapshot'))
+
+    def test_lv_get_origin(self):
+        self.assertEqual('fake-volume-1',
+                         self.vg.lv_get_origin('fake-snapshot'))
+        self.assertFalse(None, self.vg.lv_get_origin('test-volumes'))
+
     def test_activate_lv(self):
-        self.mox.StubOutWithMock(self.vg, '_execute')
-        self.vg._supports_lvchange_ignoreskipactivation = True
+        with mock.patch.object(self.vg, '_execute'):
+            self.vg._supports_lvchange_ignoreskipactivation = True
 
-        self.vg._execute('lvchange', '-a', 'y', '--yes', '-K',
-                         'fake-vg/my-lv',
-                         root_helper='sudo', run_as_root=True)
+            self.vg._execute('lvchange', '-a', 'y', '--yes', '-K',
+                             'fake-vg/my-lv',
+                             root_helper='sudo', run_as_root=True)
 
-        self.mox.ReplayAll()
-
-        self.vg.activate_lv('my-lv')
-
-        self.mox.VerifyAll()
+            self.vg.activate_lv('my-lv')
 
     def test_get_mirrored_available_capacity(self):
         self.assertEqual(2.0, self.vg.vg_mirror_free_space(1))
@@ -370,3 +380,27 @@ class BrickLvmTestCase(test.TestCase):
         self.vg.vg_name = "test-volumes"
         self.vg.extend_volume("test", "2G")
         self.assertFalse(self.vg.deactivate_lv.called)
+
+    def test_lv_deactivate(self):
+        with mock.patch.object(self.vg, '_execute'):
+            is_active_mock = mock.Mock()
+            is_active_mock.return_value = False
+            self.vg._lv_is_active = is_active_mock
+            self.vg.create_volume('test', '1G')
+            self.vg.deactivate_lv('test')
+
+    def test_lv_deactivate_timeout(self):
+        with mock.patch.object(self.vg, '_execute'):
+            is_active_mock = mock.Mock()
+            is_active_mock.return_value = True
+            self.vg._lv_is_active = is_active_mock
+            self.vg.create_volume('test', '1G')
+            self.assertRaises(exception.VolumeNotDeactivated,
+                              self.vg.deactivate_lv, 'test')
+
+
+class BrickLvmTestCaseIgnoreFDWarnings(BrickLvmTestCase):
+    def setUp(self):
+        self.configuration = mock.Mock(conf.Configuration)
+        self.configuration.lvm_suppress_fd_warnings = True
+        super(BrickLvmTestCaseIgnoreFDWarnings, self).setUp()
